#include "math.h"

// defining some contants
float hbar = 197.3;       // MeV*fm
float e2 = 1.440;         // MeV*fm
float m0 = 931.5;         // MeV/c
float r0 = 1.4;           // fm
float m  = 1E15;          // m->fm
float cm = 1E13;          // cm->fm
float mm = 1E11;          // mm->fm
float Ry = 13.8E-6;
float pi = 3.1415926;

void claski(float *AM,float &E,float &THLBO,float &THCMO,float &THLBR,
            float &THCMR,float &EO,float &ER,float &VR,float &RO,float &RR,float &Q)
{
  float RTOD=180./pi;
  float DTOR=1.0/RTOD;
  float M = m0;
  float AP=AM[0]*M;
  float AT=AM[1]*M;
  float AO=AM[2]*M;
  float AR=AM[3]*M;
  THLBO=THLBO*DTOR;
  float A0=sqrt(AO*(AO+AR-AT)/(AT*AR*(1.0+(Q/E)*(AO+AR)/AT)));
  float AAR=A0*AR/AO;
  float X=A0*sin(THLBO);
  THCMO=atan(X/sqrt(1.0-X*X))+THLBO;
  THCMR=pi-THCMO;
  THCMR=-THCMR;
  THLBR=atan(sin(THCMO)/(cos(THCMO)-AAR));
  float RAT=AO/AR*pow((sin(THLBO)/sin(THLBR)),2);
  EO=(E+Q)/(RAT+1.0);
  ER=E+Q-EO;
  VR=sqrt(1.930*ER/AR);
  RO=(A0*cos(THCMO)+1.0)*pow((sin(THLBO)/sin(THCMO)),3);
  RR=(AAR*cos(THCMO)-1.0)*pow((sin(THLBR)/sin(THCMO)),3);
  THLBO=THLBO*RTOD;
  THLBR=THLBR*RTOD;
  THCMO=THCMO*RTOD;
  THCMR=THCMR*RTOD;
  return;
}

// returns sigma rutherford in mbarns
float ruth(float *Z,float ECM, float THCM)
{
  float ZSQ=(Z[0]*Z[1])*(Z[0]*Z[1]);
  float CRUT=1.296*ZSQ;
  float CSC4=pow((1.0/sin(THCM*pi/180./2.)),4);
  float SIGRUT=CRUT*CSC4/(ECM*ECM);
  return SIGRUT;
}

// calcs interaction radius of one nucleus
float Ri(float A)                                     
{ return 1.28*pow(A,1./3.)-0.76+0.8*pow(A,-1./3.); }

// calcs radius of a nucleus
float R(float A)                                      
{ return Ri(A)*(1-1./(Ri(A)*Ri(A))); }

// calcs interaction radius between two nuclei
float Rint(float A1,float A2)                         
{ return R(A1)+R(A2)+4.49-(R(A1)+R(A2))/6.35;}

// calcs ECM
float Ecm(float Elab, float Aa, float Ap)             
{ return Elab*Aa/(Ap+Aa); }

// calcs Elab
float Elab(float Ecm, float Aa, float Ap)             
{ return Ecm*(Ap+Aa)/Aa; }

// calcs reduced mass
float Mu(float Aa, float Ap)                          
{ return m0*Aa*Ap/(Aa+Ap); }

// calcs eta
float Eta(float Zp, float Za, float Aa, float elab)   
{ return Zp*Za*sqrt(m0*Aa/(elab/Ry)); }

// calcs wave number
float K(float Aa,float Ap,float Elab)                 
{ return 0.219*Ap/(Aa+Ap)*sqrt(Aa*Elab); }

// calcs distance of closest approach
float DClosest(float Zp,float Ap, float Za, float Aa, float elab) 
{ return (0.72*Za*Zp)/elab*(Aa+Ap)/Aa;}

// calcs grazing angular momentum
float Lgrazing(float Zp,float Ap, float Za,float Aa, float elab) 
{
  float k = K(Aa,Ap,elab);
  float rint =Rint(Aa,Ap);
  float d = DClosest(Zp,Ap,Za,Aa,elab);
  return k*rint*sqrt(1-2*d/rint);
}

// calcs grazing angle
float Thetagrazing(float lgrazing, float eta)         
{ return atan(eta/lgrazing); }

