#include "log.hh"

ofstream   *__LOG__  = NULL;
TGTextView *__VIEW__ = NULL;

////////////////////////////////////////////////////////////////////////////
void openLog(char*file)
{
  closeLog();
  __LOG__ = new ofstream(file);
}
////////////////////////////////////////////////////////////////////////////
void appendLog(char*file)
{
  closeLog();
  __LOG__ = new ofstream(file, ios::out|ios::app);
}
////////////////////////////////////////////////////////////////////////////
void closeLog()
{
  if(!__LOG__) return;
  __LOG__->close();
  delete __LOG__;
  __LOG__ = NULL;
}
////////////////////////////////////////////////////////////////////////////
void   setTextView(TGTextView* view)
{
  __VIEW__ = view;
}
////////////////////////////////////////////////////////////////////////////
ofstream*   getLog()
{
  return __LOG__;
}
////////////////////////////////////////////////////////////////////////////
TGTextView* getView()
{
  return __VIEW__;
}
////////////////////////////////////////////////////////////////////////////
bool fileExists(char* file)
{
  long id,flag,mod;
#if ROOT_VERSION_CODE >= ROOT_VERSION(4,0,0)
	Long64_t size;
#else
  long size;
#endif
  if(gSystem->GetPathInfo(file,&id,&size,&flag,&mod)==0) return true;
  return false;
}
////////////////////////////////////////////////////////////////////////////
float convertToSeconds(char* line)
{
  TString a = line;  
  int p = a.Index(":");
  if(p<0) return atof(a.Data());  
  TString a1 = a(0,p);
  float b1 = atof(a1.Data());
  TString a2 = a(p+1,a.Length()-p);  
  p = a2.Index(":");
  if(p<0) return b1*60+atof(a2.Data());  
  TString a3 = a2(0,p);
  float b2 = atof(a3.Data());
  TString a4 = a2(p+1,a.Length()-p);  
  return b1*3600+b2*60+atof(a4.Data());
}
////////////////////////////////////////////////////////////////////////////
void message(int mode,char * title,char* text)
{
  int retCode;
  EMsgBoxIcon icon = kMBIconExclamation;
  if(mode==2) icon = kMBIconAsterisk;
  if(mode==3) icon = kMBIconStop;
  if(mode<=3) new TGMsgBox(gClient->GetRoot(),gClient->GetRoot(),title,text,icon,kMBOk,&retCode);
  else  cout <<text<<endl;
  return;
}
////////////////////////////////////////////////////////////////////////////
bool yesno(char* title,char* text)
{
  int retCode;
  new TGMsgBox(gClient->GetRoot(),gClient->GetRoot(),title,text,kMBIconQuestion,kMBYes+kMBNo,&retCode);
  if(retCode==kMBYes) return true;
  return false;
}
////////////////////////////////////////////////////////////////////////////
bool overwrite(char* file)
{
  TString title="File exists!";
  TString msg="Overwrite File?\n";
  msg+=file;
  return yesno((char*)title.Data(),(char*)msg.Data());
}
////////////////////////////////////////////////////////////////////////////
void log(int type,const char* va,...)
{
  char line[300];
  va_list ap;
  va_start(ap,va);
  int n = vsprintf(line,va,ap);
  TString msg = line;
  if(type!=10)
  {
    TDatime now;
    int hour = now.GetHour();
    int minute = now.GetMinute();
    int seconds = now.GetSecond();
    int day = now.GetDay();
    int month = now.GetMonth();
    int year = now.GetYear();
    sprintf(line,"%02d/%02d/%02d %02d:%02d:%02d: %s",day,month,year,hour,minute,seconds,msg.Data());  
  }
  msg = line;
  if(type==0) sprintf(line,"LOG: %s",msg.Data());
  if(type==1) sprintf(line,"WAR: %s",msg.Data());
  if(type==2) sprintf(line,"ERR: %s",msg.Data());
  
  if(__VIEW__)
  {
    __VIEW__->AddLine(line);
    int lines = __VIEW__->ReturnHeighestColHeight();
    int h = __VIEW__->GetHeight();
    if(lines>h) __VIEW__->SetVsbPosition(__VIEW__->ReturnLineCount());
  } else cout <<line<<endl;
  if(__LOG__) (*__LOG__) <<line<<endl;  
}
////////////////////////////////////////////////////////////////////////////
void log(const char* va,...)
{
  char line[300];
  va_list ap;
  va_start(ap,va);
  int n = vsprintf(line,va,ap);
  log(0,line);
}
////////////////////////////////////////////////////////////////////////////
void openHelp(char *url)
{
  int stat;
  TString system = gSystem->GetBuildArch();
  char cmd[300];
  if(system.BeginsWith("linux")) //linux
  {
    sprintf(cmd,"mozilla %s&",url);
		stat = gSystem->Exec(cmd);
  } 
  else if(system.BeginsWith("win"))
  {
    TString windir = gSystem->Getenv("windir"); 
	  sprintf(cmd,"%s\\explorer.exe %s",windir.Data(),url);
    stat = gSystem->Exec(cmd);
  }
}
