#include "UsbIO.h"
#include "TSystem.h" 

#ifndef _CINT_
#include "cycfx2dev.hh" 
#include "fx2-iolib.cc" 
#include "cycfx2dev.cc" 
#endif

#include <iostream>
using namespace std;

ClassImp(UsbIO)

/*!
Default contructor. Set object name and title
\param name is the object name
\param title is the object tile
*/
UsbIO::UsbIO(char* name,char* title):PelIO(name,title)
{
  gSystem->Exec("/sbin/rmmod usbtest");
  mIsUsb = false;
}
/*!
*/
UsbIO::~UsbIO()
{
	for(int i=0;i<MAXOUTPUT;i++) closeOutput(i);
  closeInput();
}
/*!
Open input file from stream
\param file is the file name. If NULL Usb connection is open
*/
int UsbIO::openInput(char *file)
{
  if(mInput) { delete mInput; mInput = NULL;}
	
	TString tmp = file;
	
	if(tmp.BeginsWith("/dev/usb"))
	{
		 int s;
     //resetfx2_();
     s = initfx2_();
		 if(s!=0) return kWarn;
		 mIsUsb = true;
     return kOk;

	} 
	else 
	{ 
		if(isDebug()) cout <<"Opening file instead of Usb"<<endl;
		return PelIO::openInput(file);
  }
	return kOk;
}
/*!
Close input file 
*/
int UsbIO::closeInput()
{
  if(!mIsUsb) return kWarn; 
  else
  {
    stop();
    mIsUsb = false;
  }
  return kWarn;
}
/*!
Read data block from Usb
*/
int UsbIO::readBlock()
{
  if(!mIsUsb) 
  {
    PelIO::readBlock();
  }
  else
  {
	  int arg = 0;
		int ierr = 0;
		int nbytes = 0;
		int ndat = 256;
    
		nbytes = get_fx2buff_(ndat,(unsigned char*)mBlock);
    if(isDebug()) cout <<"USB: ierr = "<<ierr<<"  nbytes = "<<nbytes<<endl;
    if(nbytes<2) return kEOF;
		
    BLOCKSIZE = nbytes/sizeof(unsigned short int);
    if(BLOCKSIZE<1) return kEOF;
    if(mDebug)
    {
      for(int i=0;i<BLOCKSIZE/8;i++) 
      { 
        for (int j=0;j<8;j++) cout <<hex<<(unsigned int)mBlock[i*8+j]<<"  "; 
        cout <<dec<<endl;
      }    
    }
  }
  return decodeBlock();
}
/*!
*/
int UsbIO::getNEventsLost()
{  
  if(!mIsUsb) return 0;
  else
  {
  }
	return 0;
}
/*!
*/
int UsbIO::getNUsedBuffers()
{  
  if(!mIsUsb) return 0;
  else
  {
  }
	return 0;
}
/*!
*/
int UsbIO::getNBuffers()
{  
  if(!mIsUsb) return 0;
  else
  {
  }
	return 0;
}
/*!
\param file is the .evo filename
*/
int UsbIO::loadEVO(char* file)
{
  if(mIsUsb)
  {  
	   TString T = file;
		 if(!T.EndsWith(".ihx")) return kWarn;
		 
		 //int s;
     //char *p="prg:fx2prog.ihx";
		 TString F = "prg:";
		 //TString F = "";
		 F+=file;
     resetfx2_();
     int s = initfx2_();
		 if(s!=0) return kWarn;
		 
     prgfx2_((char*)F.Data());
     runfx2_();
     return kOk;
  }
  return kOk;
}
/*!
*/
int UsbIO::start()
{     
  if(mIsUsb)
  {
    char *FX2CMD;
    FX2CMD="sbulk:1,R";
		//cout <<FX2CMD<<endl;
    int rv=cmd_proc_(1,FX2CMD);
    if(isDebug()) cout <<"Usb has started\n";
  } 
  else
  {
    if(isDebug()) cout <<"Usb Connection is not open \n";
    return kWarn;
  }
  return kOk;
}
/*!
*/
int UsbIO::setBufferSize(int size)
{     
  if(mIsUsb)
  {
    int arg=size; 
		int size1 = 0;
    if(isDebug()) cout <<"Usb buffer size set to "<<size1<<". Requested value was = "<<size<<"\n";
    return size1;
  } 
  else
  {
    if(isDebug()) cout <<"Usb Connection is not open \n";
    return 0;
  }
  return 0;
}
/*!
*/
int UsbIO::stop()
{     
  if(mIsUsb)
  {
    char * FX2CMD;
    FX2CMD="sbulk:1,S";
    int rv=cmd_proc_(1,FX2CMD);
    if(isDebug()) cout <<"Usb has stopped\n";
  } 
  else
  {
    if(isDebug()) cout <<"Usb Connection is not open \n";
    return kWarn;
  }
  return kOk;
}
/*!
\param slot is a slot in the Usb where the scaler module is located
\param value is an array with the scaler values
*/
int UsbIO::readScalers(int slot,int* value,int crate)
{     
  if(mIsUsb)
  {
    if(isDebug()) cout <<"Scalers are not implemented in USB \n";
    for(int i=0;i<12;i++) value[i] = 0;
    return kWarn;
  } 
  else
  {
    if(isDebug()) cout <<"Usb Connection is not open \n";
    for(int i=0;i<12;i++) value[i] = 0;
    return kWarn;
  }
  return kOk;
}
/*!
\param slot is a slot in the Usb where the scaler module is located
*/
int UsbIO::clearScalers(int slot,int crate)
{     
  if(mIsUsb)
  {
    if(isDebug()) cout <<"Scalers are not implemented in USB \n";
    return kWarn;
  } 
  else
  {
    if(isDebug()) cout <<"Usb Connection is not open \n";
    return kWarn;
  }
  return kOk;
}
