// cam_tu16.h
// ********************************************************************
//                CC16 / PC16-TURBO routines for C++
//  ----------------------------------------------------------------
//   Unit contains :
//     cam_adr         	- Set I/O Address (BADR)
//     getstatus     	- Get CC16 Status
//     cam_cratecheck	- Test crate available
//     set_crate     	- Set geographical crate Address
//     cam_controller_ini - Controller Initialization
//     cam_0         	- set CAMAC Command register to
//     cam_z         	- CAMAC Initialize (Z)
//     cam_c         	- CAMAC Clear (C)
//     cam_i         	- CAMAC Inhibit (I)
//     cam_ci        	- CAMAC Clear + Inhibit (I) setzen
//     cam_irq       	- enable CAMAC LAM-Interrupt
//     cam_noirq     	- disable CAMAC LAM-Interrupt
//     cam_q         	- get CAMAC Q-response
//     cam_x        	- get CAMAC X-response
//     cam_nfa       	- set N,F,A start CAMAC cycle
//     cam_nfa_read  	- set N,F,A and read 2 byte
//     cam_nfaqx_read  	- set N,F,A and read 2 byte with Q and X
//     cam_nfaqx_read24 - set N,F,A and read 3 byte with Q and X
//     cam_nfa_write 	- set N,F,A and write 2 byte
//     cam_nfaqx_write 	- set N,F,A and write 2 byte with Q and X
//     cam_nfaqx_write24- set N,F,A and write 2 byte with Q and X
//     cam_lam       	- get LAM source
//     cam_lamf      	- any LAM request
//
//     badr := basic I/O address
//     copyright:    A. Ruben, W-Ie-Ne-R, Plein & Baus GmbH  21.06.95
//                   revision
// ********************************************************************

/* #include <dos.h> */

  int badr00;
  int badr02;
  int badr04;
  int badr06;
  int badr08;
  int badr0a;
  int badr0c;
  int badr0e;
  int badr10;
  int badr12;
  int badr14;
  int badr16;
  int badr18;
  int badr1a;
  int badr1c;
  int badr1e;

void   outport(int,int);
int inport(int);
void gettxt(char *);
void cam_adr(int);
int  cam_cratecheck (void);
void set_crate (int);
void cam_controller_ini (int);
int  getstatus(void);
void cam_z(void);
void cam_c(void);
void cam_i(void);
void cam_ci(void);
void cam_0(void);
int  cam_q(void);
int  cam_x(void);
void cam_irq(void);
void cam_noirq(void);
void cam_nfa(int, int, int);
void cam_nfa_write(int, int, int, int);
void cam_nfaqx_write(int, int, int, int, int, int);
void cam_nfaqx_write24(int, int, int, long int, int, int);
unsigned int  cam_nfa_read(int, int, int);
unsigned int  cam_nfaqx_read(int, int, int, int, int);
long int cam_nfaqx_read24(int, int, int, int, int);

//**************************************************************************}

void cam_adr(int bad)
{
  badr00=bad;
  badr02=bad+2;
  badr04=bad+4;
  badr06=bad+6;
  badr08=bad+8;
  badr0a=bad+10;
  badr0c=bad+12;
  badr0e=bad+14;
  badr10=bad+16;
  badr12=bad+18;
  badr14=bad+20;
  badr16=bad+22;
  badr18=bad+24;
  badr1a=bad+26;
  badr1c=bad+28;
  badr1e=bad+30;
};
//**************************************************************************}

int  cam_cratecheck (void)
{
  int j;
  int cnr;
  cnr=0;
  outport(badr00,cnr);
  j=inport(badr10);
  j=(j & 4)-4;
  if (j<0)
  {
   j=1;
  }
  return(j);
};
//**************************************************************************}

void   set_crate (int crate)
{
  outport(badr00,(crate & 16));
};
//**************************************************************************}

void cam_controller_ini (int bad)
{
  cam_adr(bad);
  set_crate(0);
  cam_z();
};
//**************************************************************************}

int getstatus(void)
{
  char i;
  char j;
  j=0;
  j=inport(badr10);
  return(j & 7);
};
// ***********************************************************************}

void cam_z(void)
{
  outport(badr16,0);
};
// **************************************************************************}

void cam_c(void)
{
  outport(badr14,0);
};
// *************************************************************************}

void cam_i(void)
{
  outport(badr0e,10);
};
// ************************************************************************}

void cam_ci(void)
{
  outport(badr14,0);
  outport(badr0e,10);
};
// *************************************************************************}

void cam_0(void)
{
  outport(badr0e,11);
};
// *************************************************************************}

int cam_q(void)
{
  char j;
  j=inport(badr12);
  return ( j & 1);
};
// ************************************************************************}

int cam_x(void)
{
  char j;
  j=inport(badr12);
  return ( j & 2) > 1;
};
// ************************************************************************}

void cam_irq(void)
{
  outport(badr0e,12);
};
// *************************************************************************}

void cam_noirq(void)
{
  outport(badr0e,13);
};

// *************************************************************************}

void cam_nfa(int n, int f, int a)
{ int nfa;
  nfa= a << 5;
  nfa= (nfa | f) << 5;
  nfa= nfa | (n+0x8000);
  outport(badr02,nfa);
};
// *************************************************************************}

void cam_nfa_write(int n, int f, int a, int data)
{ int nfa;
  nfa= a << 5;
  nfa= (nfa | f) << 5;
  nfa= nfa | n;
  outport(badr02,nfa);
  outport(badr06,data);
};
// *************************************************************************}

void cam_nfaqx_write(int n, int f, int a, int data,int *q, int *x)
{ int nfa;
  char j;
  nfa= a << 5;
  nfa= (nfa | f) << 5;
  nfa= nfa | n;
  outport(badr02,nfa);
  outport(badr06,data);
  j=inport(badr12);
  *q=( j & 1);
  *x=( j & 2) > 1;
};
// *************************************************************************}

void cam_nfaqx_write24(int n, int f, int a, long int data,int *q, int *x)
{ int nfa;
  char j;
  int ih;
  int il;
  ih= (int) ((data >> 16) & 255);
  il= (int) data;
  nfa= a << 5;
  nfa= (nfa | f) << 5;
  nfa= nfa | n;
  outport(badr02,nfa);
  outport(badr04,ih);
  outport(badr06,il);
  j=inport(badr12);
  *q=( j & 1);
  *x=( j & 2) > 1;
};
// *************************************************************************}

unsigned int cam_nfa_read(int n, int f, int a)
{ int nfa;
  nfa= a << 5;
  nfa= (nfa | f) << 5;
  nfa= nfa | (n+0x8000);
  outport(badr02,nfa);
  return ((unsigned int) inport(badr0a));
};
// *************************************************************************}

unsigned int cam_nfaqx_read(int n, int f, int a, int *q, int *x)
{ int nfa;
  char j;
  nfa= a << 5;
  nfa= (nfa | f) << 5;
  nfa= nfa | (n+0x8000);
  outport(badr02,nfa);
  nfa=(unsigned int) inport(badr0a);
  j=inport(badr12);
  *q=( j & 1);
  *x=( j & 2) > 1;
  return (nfa);
};
// *************************************************************************}

long int cam_nfaqx_read24(int n, int f, int a, int *q, int *x)
{ int nfa;
  char j;
  long int lh;
  nfa= a << 5;
  nfa= (nfa | f) << 5;
  nfa= nfa | (n+0x8000);
  outport(badr02,nfa);
  lh=inport(badr08) & 255;
  lh=lh <<16;
  lh += (unsigned int) inport(badr0a);
  j=inport(badr12);
  *q=( j & 1);
  *x=( j & 2) > 1;
  return (lh);
};
// *************************************************************************}

int cam_lam(int station)
{
  cam_nfa(station,8,0);
  return (cam_q());
};
// *************************************************************************}

int cam_lamf(void)
{
  unsigned int j;
  j=inport(badr12);
  j= j & 4;
  return(j > 2);
};

void   outport(int w,int op)
{

}

int inport(int w)
{
}

void gettxt(char text[])
{
int i;
int MAX_SEND_SIZE = 130;
char c;

        i=0;
        while(i<MAX_SEND_SIZE && (c!='\n')){
          c=getchar();
          text[i++]=c;
        }
        i--;
        text[i]=0;
}
 
