/*
 *    c111at-drv.c   << Device Driver for C111A + A151A Turbo>> 26 Jan. 2000
 *    (based on  camdrv.c - A Device Driver for DSP CC)
 *    Special feature: comand interpreter for the Event Handler evo object
 *    files.

    Copyright (C) 2000 R.V. Ribas - Universidade de S� Paulo, Inst. de F�ica

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*/
	#define ZEROCRATE 1
	#define CLEARCRATE 2
	#define SWITCHLAM 3
	#define CRATEONLINE 4
	#define CRATEOFFLINE 5
	#define CRATESTATUS 6
  #define CCGETLOST    27
  #define CCSETCRATE   28
  #define CCGETUSED    29
  #define CCGETBUF     30

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/ioctl.h>
#include <linux/ioport.h>
#include <linux/fs.h>
#include <linux/interrupt.h>
#include <linux/sched.h>
#include <linux/errno.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/delay.h>
#include "c111at-drv.h"


MODULE_LICENSE("GPL");
MODULE_AUTHOR("R. V. Ribas");
MODULE_DESCRIPTION("Driver for CC ISA CAEN C111A");

#define BUFFER_SIZE 16384
#define INSTR_SIZE  4096
#define ENABLE 0
#define DISABLE 4

static int iev=0,MAXSIZE=512,FLAG[16],NEXTW,NEXTR,NBUF,nwords=0;
static int Running=0,lost=0,lock=0;
static unsigned int offset_b=0;
/*
static volatile unsigned short int cc_buf[BUFFER_SIZE];
static volatile unsigned int  cc_inst_buf[INSTR_SIZE]; 
*/
static unsigned short int cc_buf[BUFFER_SIZE];
static unsigned int  cc_inst_buf[INSTR_SIZE]; 

static int cc_open(struct inode* inode, struct file* filep);
static ssize_t cc_read (struct file *, char *, size_t,loff_t *);
static ssize_t cc_write (struct file *, const char *, size_t, loff_t *);
static int cc_release(struct inode* inode, struct file* filep);
static int cc_ioctl(struct inode *inode, struct file *filep, unsigned int cmd, unsigned long arg);
irqreturn_t cc_interrupt(int irq, void* dev_id, struct pt_regs* regs);

/*
 * Here is the file_operation structure
 */

static struct file_operations cc_fops = {
    .owner = THIS_MODULE,
    .open = cc_open,
    .write = cc_write,
    .read = cc_read,
    .release = cc_release,
    .ioctl = cc_ioctl,
};

int cc_init(void);
void cc_exit(void);


/*
 * The following functions are only internal used; they contain the
 * I/O code
 */

static void set_crate_number(int);
static void camcl(int );
static int camo24 (int , int);
static int cami16 (int , int *);
static int ev_read (unsigned short *);
static int cami24 (int , int *);
static int camo16 (int , int);
static int NFA, CC_IO_OP;
static int major = AC_MAJOR;

struct wait_queue *cc_wait_q;
static int cc_in_use[4];

module_init(cc_init);
module_exit(cc_exit);

int  __init cc_init(void)
{
  int i;

  if (register_chrdev(major, CC_NAME, &cc_fops))
  {
    printk(" unable to get major %i for " CC_NAME " device\n",major);
    return -EIO;
  }
  for(i=0;i<4;i++){
    printk("===============================\nSetting up for crate # %d \n",i);
    cc_in_use[i] = 0;
    set_crate_number(i);
    camcl(4);
    camcl(1);
    camcl(2);
    camcl(4);
    camcl(DISABLE);
    camcl(5);
  }
  printk("===============================\n");
  printk (CC_NAME ": initializing module \n");
  return 0;
}

void __exit cc_exit(void)
{
  unregister_chrdev(AC_MAJOR, CC_NAME);
  free_irq(CC_IRQ,NULL);
  release_region(GEO_port,0x20);
  printk(CC_NAME ": module removed\n");
  return;
}

static int cc_open (struct inode *inode, struct file *file)
{
 unsigned int crate = MINOR(inode->i_rdev);
 int istat;
  printk("Opening Crate # %d \n",crate);
  if (crate > 3){
    printk("Crate # must be <4\n");
    return -EBUSY;
    }
  if(cc_in_use[crate]){
    printk("Crate in Use...\n");
    return -EBUSY;
    }
  if(!request_region(GEO_port,0x20,"C111aT")){
   printk(CC_NAME": Unable to register CAMAC ports\n");
   return -EIO;
  }

  istat = request_irq (CC_IRQ, cc_interrupt,IRQF_DISABLED, "C111aT" ,NULL);

/*
int request_irq(unsigned int irq, irq_handler_t handler,
		unsigned long irqflags, const char *devname, void *dev_id)
*/

  if (istat){
    printk(CC_NAME": can't allocate IRQ%d for DSP camac\n",CC_IRQ);
    release_region(GEO_port,0x20);
    return -EIO;
  }

  cc_in_use[crate] = 1;
  set_crate_number(crate);
  camcl(1);
  camcl(2);
  camcl(4);
  printk("Crate open OK  \n");
  return 0;
}

static int cc_release (struct inode *inode, struct file *file)
{
 unsigned int crate = MINOR(inode->i_rdev);
 cc_in_use[crate] = 0;
 release_region(GEO_port,0x20);
 free_irq(CC_IRQ,NULL);
 camcl(1);
 camcl(5);

 return 0;
}

irqreturn_t cc_interrupt (int irq, void *dev, struct pt_regs *regs)

{
  static unsigned short ev_buf[128];
  int i;
  iev=0;
  lock=0;
  ev_read(ev_buf);
  nwords=nwords+iev;
  if(nwords > (MAXSIZE-64)){
    FLAG[NEXTW++]=nwords;
    if(NEXTW >= NBUF) NEXTW=0;
    nwords=0;
    offset_b=NEXTW*MAXSIZE;
  }
  if(FLAG[NEXTW]==0){
    for(i=0;i<iev;i++)cc_buf[offset_b+i]=ev_buf[i];
    offset_b=offset_b+iev;
  } else lost++;

  local_irq_enable();
  return IRQ_HANDLED;
}

static int cc_ioctl(struct inode * inode, struct file * file ,
              unsigned int cmd,  unsigned long arg)
{
  int crate = MINOR(inode->i_rdev);
  int i;
  unsigned short int dummy[64];
  if (crate > 3)
    return -ENODEV;

/* cmd validos: CCREAD16, CCREAD24, CCWRITE16, CCWRITE24, CCRESET 
                CCREADEVNT CCWAITLAM CCMAXSIZE
*/
  switch (cmd) {
  case CCMAXSIZE:
    MAXSIZE=arg;
    if(MAXSIZE<256)MAXSIZE=256;
    if(MAXSIZE>4096)MAXSIZE=4096;
    NBUF=BUFFER_SIZE/MAXSIZE;
    if(NBUF > 16) NBUF=16;
    printk(" maxsize,nbuf = %d %d\n",MAXSIZE,NBUF);
    return MAXSIZE;
    break;
  case CCRESET:
    if(arg == ENABLE){
      lost=0;
      printk(" Running...\n");
      for(i=0;i<16;i++)FLAG[i]=0;
      nwords=0;
      offset_b=0;
      for(i=0;i<BUFFER_SIZE;i++)cc_buf[i]=0;
      NEXTW=0;
      NEXTR=0;
      Running = 1;
      ev_read(dummy);
    }
    if(arg==DISABLE)printk(" Stopped...lost %d events\n",lost);
    set_crate_number(crate); 
    camcl ((int) arg);
    return 0;
    break;
  case CCWAITLAM:
    printk("Invalid call: CCWAITLAM \n");
    return -1;
    break;
  case CCGETLOST:
      //return lost;
      return 0;
      break;
  case CCGETUSED:
      //return NBUFFULL;
      return 0;
      break;
  case CCGETBUF:
      //return NBUF;
      return 0;
      break;
  case CCSETCRATE:
      if(arg<4) set_crate_number(arg);
      return 0;
  default:
    NFA = arg;
    CC_IO_OP=cmd;
    return 0;
  }
}
static ssize_t
cc_read (struct file *file, char *buffer, size_t size, loff_t *loff)
{
  int result;
  int max,nbytes,QX;
  unsigned int Dat;
  unsigned short int *offset;
  unsigned short int dummy[64];

#ifdef RTLINUX
  rtl_irqstate_t irq_state;
#endif
 
  /*  Check that we can really write to the buffer   */
  max=size>>1;
  if(max>iev) max=iev;
  if(max > BUFFER_SIZE){
   printk(CC_NAME": transfer block larger than 8K %d\n",size);
   return -1;
  }
  if (!(result = access_ok(VERIFY_WRITE,buffer,size))) {
     printk(" VERIFY_WRITE Fail\n");
     return result;
  }

  switch (CC_IO_OP){
  case CCREAD16:
    QX = cami16 (NFA,&Dat);
    result=copy_to_user(buffer, &Dat, 2);
    return QX;
    break;
  case CCREAD24:
    QX = cami24 (NFA,&Dat);
    result=copy_to_user(buffer, &Dat, 4);
    return QX;
    break;

  case CCREADEVNT:
    nbytes = FLAG[NEXTR]*2;
    if(nbytes > 0){
      offset = (short int *)cc_buf;
      FLAG[NEXTR]=0;
      offset=offset+MAXSIZE*NEXTR++;
      if(NEXTR >= NBUF) NEXTR = 0;
      result=copy_to_user(buffer, (void *)offset,nbytes);
      return nbytes;
    }else {
      lock++;
      if(lock > 20) {
	ev_read(dummy);
	lock=0;
        }
      return 0;
    }
    break;

  default:
    return -1;
  }
}

static ssize_t
cc_write (struct file *file, const char *buffer, size_t size, loff_t *loff)
{
  int result, Dat=0;
  unsigned short int dummy[64];


  /*
   * Check that we can really read from the buffer
   */
  
  if (!(result = access_ok(VERIFY_READ,buffer,size)))
    return result;


  switch (CC_IO_OP){
  case CCWRITE16:
    /*    Dat=get_fs_long(buffer); */
    result=copy_from_user(&Dat,buffer,2);
    return camo16(NFA,Dat);
    break;
  case CCWRITE24:
    /*    Dat=get_fs_long(buffer); */
    result=copy_from_user(&Dat,buffer,4);
    return camo24(NFA,Dat);
    break;
  case CCLOADEH:
    result=copy_from_user((int *)cc_inst_buf,buffer,size);
    printk("loading %d instructions \n",size/4);
    iev=0;
    ev_read(dummy);
    return 0;
    break;
  default:
    return -1;
  }
}

static int camo16(int NFA,int D)
{
  outw(NFA,FNA_port);
  outw(D,LW_W);
  return inb(R_S) & 3;
}

static int camo24(int NFA,int D)
{
  int BH;
  BH = (int) ((D >> 16) & 255); 
  outw(NFA,FNA_port);
  outw(D,LW_W);
  return inb(R_S) & 3;
}
static int cami16(int NFA,int *D)
{
  int lw;
  NFA = NFA |0x8000;
  outw_p(NFA,FNA_port);
  lw = (inw(LW_R)&0xffff);
  udelay(1);
  *D = lw;
  return (inb(R_S) & 3);
}

static int cami24(int NFA, int *D)
{
  unsigned int lw,hb;
  NFA = NFA |0x8000;
  outw_p(NFA,FNA_port);
  hb = inb(HB_R)&0x00ff;
  hb = inb(HB_R)&0x00ff;
  lw = inw(LW_R);
  lw = lw + (hb<<16);
  *D = lw;
  udelay(1);
  return inb(R_S) & 3;
}

static void camcl(int I)
{
   int bits, opt;
   unsigned char st;
   opt=(0x000f&I);
   bits=(0x0ff00&I)>>8;
   switch(opt){
     case ZEROCRATE:
       outb(0,Crate_Z);
       printk("Z on crate \n");
       break;
     case CLEARCRATE:
       outb(0,Crate_C);
       printk("Clear on crate \n");
       break;
     case SWITCHLAM:
       if(I==ENABLE)printk("Enable LAM\n");
       if(I==DISABLE)printk("Enable LAM\n");
       outb(bits,S_mod_W);
       break;
     case CRATEONLINE:
       outb(0,Crate_on);
       bits=0x000B;
       outb(bits,S_mod_W);
       printk("Crate On-line, Clear Inhibit \n");
       break;
     case CRATEOFFLINE:
       bits=0x000A;
       outb(bits,S_mod_W);
       outb(0,Crate_off);
       printk("Crate Off-line, Set Inhibit \n");
       break;
     case CRATESTATUS:
       st=inb(R_S)&0x07;
       printk("Crate Status: %x \n",st);
     default:
     break;
   }
}

static void set_crate_number(int I)
{
  outw(I,GEO_port);
  printk(" Setting up for crate # %d \n",I);
}

static int ev_read (unsigned short int ev_buf[])
{
  int i,looping,lw;
  unsigned short int par,PAT=0,EX=0,TXR=0;
  unsigned short int CA=0;
  unsigned int inst;
  int hbit;
  looping=1;
  i=0;
  while ((looping == 1) && (i<4096) ){
    hbit=(cc_inst_buf[i] & 0x00f00000);
    if(hbit>0x200000){
      inst = (cc_inst_buf[i] & 0x007f0000);
      par  = cc_inst_buf[i++] & 0xffff;
    }
    else{
      inst = (cc_inst_buf[i] & 0x007fc000);
      par  = cc_inst_buf[i++] & 0x3fff;
    }
    if(inst == 0x5C0000) par = par|0x0fff;
    if((inst == 0x100000)){
      if((par&0x03e0) < 0x00e0){
        par = (par | 0x8000);
        outw_p(par,FNA_port);
        CA = inw_p(LW_R);            /* NAF - read */
      }
      else{ 
        outw_p(par,FNA_port);
        lw = 0x00ffff & CA;
        outw_p(lw,LW_W);             /* NAF - write */
      }
    }
    else if(inst == 0x200000)looping=0;               /* RTRN */
    else if(inst == 0x250000)i = par-1;               /* BRU A */
    else if(inst == 0x2A0000)looping=0;               /* INIT */
    else if(inst == 0x300000){if((PAT&par) != 0)i++;} /* SKIP PAT .ANY.# */
    else if(inst == 0x320000){if((EX&par) != 0)i++;}  /* SKIP EX .ANY.# */
    else if(inst == 0x330000){if((CA&par) != 0)i++;}  /* SKIP CA .ANY.# */
    else if(inst == 0x340000){if((PAT&par) == 0)i++;} /* SKIP PAT .NONE.# */
    else if(inst == 0x360000){if((EX&par) == 0)i++;}  /* SKIP EX .NONE.# */
    else if(inst == 0x370000){if((CA&par) == 0)i++;}  /* SKIP CA .NONE.# */
    else if(inst == 0x3b0000){if(CA<par) i++;}        /* SKIP CA .LT.# */
    else if(inst == 0x3f0000){if(CA>par) i++;}        /* SKIP CA .GT.# */
    else if(inst == 0x380000){if(PAT<par) i++;}       /* SKIP PAT .LT.# */
    else if(inst == 0x3c0000){if(PAT>par) i++;}       /* SKIP PAT .GT.# */
    else if(inst == 0x400000)CA = par;                /* MOV #,CA */
    else if(inst == 0x410000)PAT = CA;                /* MOV CA,PAT */
    else if(inst == 0x420000)CA = PAT;                /* MOV PAT,CA */
    else if(inst == 0x430000)EX = par;                /* SETX # */
    else if(inst == 0x440000)EX = EX + par;           /* ADDX # */
    else if(inst == 0x450000)CA = CA >> par;          /* SHFT # */
    else if(inst == 0x510000)TXR = CA;                /* MOV CA1,TXR */
    else if(inst == 0x580000)ev_buf[iev++] = par;     /* OUT # */
    else if(inst == 0x590000)ev_buf[iev++] = (CA&0x0fff); /* OUT CA */
    else if(inst == 0x5A0000)ev_buf[iev++] = PAT;     /* OUT PAT */
    else if(inst == 0x5C0000)ev_buf[iev++]=(TXR&par); /* MERG #,TXR */
    else if(inst == 0x600000) udelay(par&0x00ff);     /* DLAY # */
    else if(inst == 0x700000) par = 0;                /* CLRB */
    else if(inst == 0x710000) par = 0;                /* SETB */
    else if(inst == 0x000000) return iev;             /* RETURN */
    else {
     printk(" Invalid instruction \n");
     printk("inst, par: %x %x %x \n",inst,par,cc_inst_buf[i-1]);
    }
  }
  return iev;
}
MODULE_LICENSE("GPL");
