// ****************************************************************************
// Copyright (C) 2004  Alexandre Suaide
//
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// The Crate Controler Wiener-CC32 is not authorized for use as
// critical component in life support systems without the express
// written approval of ARW Elektronik Germany.
//
// Maintainer(s): Alexandre Suaide (suaide@if.usp.br)
//
// ****************************************************************************

#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/fs.h>

#include "cameh.h"
#include "common/common.c"

static int cc_in_use[MAX_CRATES];

/***************************************************************/
#ifdef KERNEL26
int __init init_module(void)
#else
static int init_module(void)
#endif
{
  int i;
  
  if (register_chrdev(AC_MAJOR, CC_NAME, &cc_fops))
  {
    printk(" unable to get major %i for " CC_NAME " device\n",AC_MAJOR);
    return -EIO;
  }
  for(i=0;i<MAX_CRATES;i++) 
  {
    printk("===============================\nSetting up for crate # %d \n",i);
    cc_in_use[i] = 0;
  }
  release_region(CAMAC,16);

  free_irq(CC_IRQ,NULL);

  printk (CC_NAME ": initializing module \n");
  return 0;
}
/***************************************************************/
#ifdef KERNEL26
void __exit cleanup_module(void)
#else
static void cleanup_module(void)
#endif
{
  if (unregister_chrdev(AC_MAJOR, CC_NAME) != 0)
    printk(CC_NAME ": cleanup_module failed\n");
  else
  
  free_irq(CC_IRQ,NULL);
  release_region(CAMAC,16);
  printk(CC_NAME ": module removed\n");
}
/***************************************************************/
static int cc_open (struct inode *inode, struct file *file)
{
 int istat,i;
 init_module();
 
  #ifndef KERNEL26
    if(check_region(CAMAC,16))
    {
     printk(CC_NAME": Unable to register CAMAC ports\n");
     return -EIO;
    }
  #endif
  
  request_region(CAMAC,16,"CAMDRV");
  istat = request_irq (CC_IRQ, cc_interrupt,SA_INTERRUPT, "CAMDRV" ,NULL);
  
  if (istat)
  {
    printk(CC_NAME": can't allocate IRQ%d for DSP camac\n",CC_IRQ);
    release_region(CAMAC,16);
    return -EIO;
  }
  for(i=0;i<MAX_CRATES;i++)
  {
    init_crate(i);
  }
  if(cc_in_use[0] == 1)
  {
    printk("===============================\n");
    printk("Crate number 0 set as current\n");
    set_crate_number(0);
    camcl(64);
    camcl(4);
  }
  return 0;
}
/***************************************************************/
static void init_crate(int crate)
{
  if (crate>=MAX_CRATES)
  {
    //printk("Crate must be smaller than $d \n",MAX_CRATES);
    return;
  }
  if(cc_in_use[crate]==1)
  {
    printk("Crate already in Use...\n");
    return;
  }
  printk("===============================\n");
  printk("Initializing crate number %d \n",crate);
  cc_in_use[crate] = 1;
  set_crate_number(crate);
  camcl(64);
  camcl(4);
  return;
}
/***************************************************************/
static void close_crate(int crate)
{
  cc_in_use[crate] = 0;
  set_crate_number(crate);
  camcl(1);
  camcl(5);
  return;
}
/***************************************************************/
static int cc_release (struct inode *inode, struct file *file)
{
  unsigned int crate = MINOR(inode->i_rdev);
  cc_in_use[crate] = 0;
  release_region(CAMAC,16);
  #ifndef KERNEL26
    free_irq(CC_IRQ,NULL);
  #endif
  close_crate(1);
  return 0;
}
/***************************************************************/
#ifdef KERNEL26
static irqreturn_t cc_interrupt(int irq, void *dev, struct pt_regs *regs)
#else
static void  cc_interrupt(int irq, void *dev, struct pt_regs *regs)
#endif
{
  read_event_now();
  #ifdef KERNEL26
    return 0;
  #else
    sti();
    return;
  #endif
}
/***************************************************************/
static void camcl(int I)
{
   
   printk("CAMCL = %d, &3 = %d\n",I,I&3);
   outb((short) I,CAM6);
   if((I & 3) != 0) outb((short) I,CAM7);
   if(I==ENABLE)
   {
   	
   }
}
